@@ -58,9 +58,8 @@ impl<'a> Iterator for OutlierNeighborIter<'a> {
     fn next(&mut self) -> Option<Self::Item> {
         let next = self
             .hive
-            .cell_neighboring_positions(self.curr_pos)
-            .into_iter()
-            .filter(|&p| !self.prev_poss.contains(&p) && self.hive.get_cell(p).is_none())
+            .cell_neighboring_free_positions(self.curr_pos)
+            .filter(|&p| !self.prev_poss.contains(&p))
             .filter(|&p| {
                 self.hive
                     .cell_neighbors(p)
@@ -68,9 +67,7 @@ impl<'a> Iterator for OutlierNeighborIter<'a> {
                     .filter(|&p| p != self.curr_pos)
                     .filter(|&p| {
                         self.hive
-                            .cells
-                            .iter()
-                            .map(|c| c.position)
+                            .all_positions()
                             .filter(|p2| !self.prev_poss.contains(p2))
                             .any(|p2| p2 == p)
                     })
@@ -105,11 +102,7 @@ impl Hive {
     }
 
     fn remove_cell(&mut self, pos: Position) {
-        let idx = self
-            .cells
-            .iter()
-            .position(|cell| cell.position == pos)
-            .unwrap();
+        let idx = self.all_positions().position(|p| p == pos).unwrap();
         self.cells.remove(idx);
     }
 
@@ -125,7 +118,16 @@ impl Hive {
         neighboring_positions(pos)
     }
 
-    fn cell_neighbors<'a>(&'a self, pos: Position) -> impl Iterator<Item = &Cell> + 'a {
+    pub fn cell_neighboring_free_positions(
+        &self,
+        pos: Position,
+    ) -> impl Iterator<Item = Position> + '_ {
+        self.cell_neighboring_positions(pos)
+            .into_iter()
+            .filter(|&p| self.get_cell(p).is_none())
+    }
+
+    fn cell_neighbors(&self, pos: Position) -> impl Iterator<Item = &Cell> + '_ {
         let neighbor_positions = self.cell_neighboring_positions(pos);
 
         self.cells
@@ -133,10 +135,14 @@ impl Hive {
             .filter(move |cell| neighbor_positions.contains(&cell.position))
     }
 
-    fn all_pieces<'a>(&'a self) -> impl Iterator<Item = &Piece> + 'a {
+    fn all_pieces(&self) -> impl Iterator<Item = &Piece> + '_ {
         self.cells.iter().flat_map(|c| c.pieces.iter())
     }
 
+    pub fn all_positions(&self) -> impl Iterator<Item = Position> + '_ {
+        self.cells.iter().map(|c| c.position)
+    }
+
     fn num_neighbors(&self, pos: Position) -> usize {
         self.cell_neighbors(pos).count()
     }
@@ -149,20 +155,19 @@ impl Hive {
     ) -> bool {
         assert!(are_neighbors(source, target)); // should be run on neighbors
 
-        let mut neighbors1 = self
+        let neighbors1 = self
             .cell_neighbors(source)
             .into_iter()
             .map(|cell| cell.position)
-            .filter(|pos| Some(*pos) != ignore)
+            .filter(|&pos| pos != target && Some(pos) != ignore)
             .collect::<HashSet<_>>();
-        neighbors1.remove(&target);
-        let mut neighbors2 = self
+
+        let neighbors2 = self
             .cell_neighbors(target)
             .into_iter()
             .map(|cell| cell.position)
-            .filter(|pos| Some(*pos) != ignore)
+            .filter(|&pos| pos != source && Some(pos) != ignore)
             .collect::<HashSet<_>>();
-        neighbors2.remove(&source);
 
         neighbors1.intersection(&neighbors2).count() != 2
     }
@@ -194,25 +199,18 @@ impl Hive {
         }
 
         if !self
-            .cells
-            .iter()
-            .map(|cell| &cell.pieces)
-            .any(|pcs| pcs.contains(&Piece::new(player_color, Species::QueenBee)))
+            .all_pieces()
+            .any(|piece| piece == &Piece::new(player_color, Species::QueenBee))
         {
             return Err(MoveError::QueenBeeNotPlaced);
         }
 
-        let cell = if let Some(cell) = self.get_cell(*path.first().unwrap()) {
+        let cell = if let Some(cell) = self.get_cell(path[0]) {
             cell
         } else {
             return Err(MoveError::MissingPiece);
         };
-
-        let piece = if let Some(piece) = cell.pieces.last() {
-            piece
-        } else {
-            return Err(MoveError::MissingPiece);
-        };
+        let piece = cell.top_piece();
 
         if piece.color != player_color {
             return Err(MoveError::WrongPlayer);
@@ -224,35 +222,58 @@ impl Hive {
 
         let is_cell_flat = cell.pieces.len() == 1;
 
+        // check if the whole path neighbors at least one of the existing ones
+        // exclude descending pieces, as they can't break this rule
+        if is_cell_flat
+            && path.iter().skip(1).any(|pos| {
+                self.cell_neighbors(*pos)
+                    .filter(|c| c.position != path[0])
+                    .count()
+                    == 0
+            })
+        {
+            return Err(MoveError::NotAdjacentToAnotherPiece);
+        }
+
         // check if the one hive rule holds wrt the piece itself
-        if self.cell_neighbors(path[0]).count() == 1 && {
-            let mut next_move_neighbors = self
-                .cell_neighbors(path[1])
-                .filter(|c| c.position != path[0]);
-            let first_neighbor = next_move_neighbors.next();
-            if next_move_neighbors.next().is_some() {
-                false
-            } else if let Some(neighbor) = first_neighbor {
-                !self
-                    .cell_neighbors(path[0])
-                    .any(|c| c.position == neighbor.position)
-                    && self.cell_neighbors(neighbor.position).count() == 1
-            } else {
-                false
+        // exclude hopping, descending, and climbing  pieces, as those can't
+        // break the one hive rule themself (only by breaking up the rest of the hive)
+        if piece.species != Species::Grasshopper && is_cell_flat && self.get_cell(path[1]).is_none()
+        {
+            for window in path.windows(2) {
+                if let &[pos1, pos2] = window {
+                    let move_direction = self
+                        .cell_neighboring_positions(pos1)
+                        .into_iter()
+                        .position(|pos| pos == pos2)
+                        .unwrap();
+                    let neighboring_directions = match move_direction {
+                        0 => [5, 1],
+                        5 => [4, 0],
+                        _ => [move_direction - 1, move_direction + 1],
+                    };
+                    let path_neighbors = neighboring_directions
+                        .into_iter()
+                        .map(|dir| DirectionalNeighborIter::new(pos1, dir).next().unwrap());
+
+                    if path_neighbors
+                        .filter(|&pos| pos != path[0]) // exclude the first piece (as it moves from its place)
+                        .all(|pos| self.get_cell(pos).is_none())
+                    {
+                        return Err(MoveError::OneHiveRuleViolation);
+                    }
+                } else {
+                    unreachable!();
+                }
             }
-        } {
-            return Err(MoveError::OneHiveRuleViolation);
         }
 
         // check if the one hive rule holds wrt the rest of the hive
+        // exclude descending pieces, as they can't break the hive
         if is_cell_flat {
-            let potential_gap = path[0];
-
             let cell_positions = self
-                .cells
-                .iter()
-                .map(|cell| cell.position)
-                .filter(|&pos| pos != potential_gap)
+                .all_positions()
+                .filter(|&pos| pos != path[0])
                 .collect::<Vec<_>>();
 
             if !are_grouped(&cell_positions) {
@@ -260,20 +281,6 @@ impl Hive {
             }
         }
 
-        // check if the whole path neighbors at least one of the existing ones
-        if path.iter().skip(1).any(|pos| {
-            if is_cell_flat {
-                self.cell_neighbors(*pos)
-                    .filter(|&cell| cell.position != path[0])
-                    .count()
-                    == 0
-            } else {
-                self.num_neighbors(*pos) == 0
-            }
-        }) {
-            return Err(MoveError::NotAdjacentToAnotherPiece);
-        }
-
         Ok(())
     }
 
@@ -308,11 +315,14 @@ impl Hive {
     }
 
     fn can_beetle_move(&self, path: &[Position]) -> Result<(), MoveError> {
-        if path.len() != 2 || !are_neighbors(path[0], path[1]) {
+        if path.len() != 2 {
             return Err(MoveError::TooLong);
         }
 
-        if self.get_cell(path[1]).is_none() && !self.has_freedom_to_move(path[0], path[1], None) {
+        if self.get_cell(path[1]).is_none()
+            && self.get_cell(path[0]).unwrap().pieces.len() == 1
+            && !self.has_freedom_to_move(path[0], path[1], None)
+        {
             return Err(MoveError::NoFreedomToMove);
         }
 
@@ -370,7 +380,7 @@ impl Hive {
     }
 
     fn can_queen_bee_move(&self, path: &[Position]) -> Result<(), MoveError> {
-        if path.len() != 2 || !are_neighbors(path[0], path[1]) {
+        if path.len() != 2 {
             return Err(MoveError::TooLong);
         }
 
@@ -496,10 +506,8 @@ impl Hive {
             .count()
             == 3
             && !self
-                .cells
-                .iter()
-                .map(|cell| &cell.pieces)
-                .any(|pcs| pcs.contains(&Piece::new(piece.color, Species::QueenBee)))
+                .all_pieces()
+                .any(|p| p == &Piece::new(piece.color, Species::QueenBee))
             && piece.species != Species::QueenBee
         {
             return Err(PlacementError::MustPlaceQueenBee);
@@ -545,25 +553,24 @@ impl Hive {
     }
 
     fn possible_queen_bee_move_paths(&self, pos: Position) -> Vec<Vec<Position>> {
-        self.cell_neighboring_positions(pos)
-            .into_iter()
-            .filter(|&p| self.get_cell(p).is_none())
+        self.cell_neighboring_free_positions(pos)
             .map(|dest| vec![pos, dest])
-            .filter(|path| self.can_move(path).is_ok())
+            .filter(|path| {
+                // "hack": don't use `Hive::can_move`, otherwise the spider's paths
+                // (that are sourced from the soldier ant, which uses queen's paths
+                // as seeds) will not generate correctly
+                let color = self.cell_color(path[0]).unwrap();
+                self.run_common_move_checks(color, path).is_ok()
+                    && self.can_queen_bee_move(path).is_ok()
+            })
             .collect()
     }
 
-    fn possible_soldier_ant_move_paths(&self, pos: Position) -> Vec<Vec<Position>> {
+    fn possible_soldier_ant_move_paths_with_duplicates(&self, pos: Position) -> Vec<Vec<Position>> {
         let mut possible_paths: Vec<_> = self
-            .cell_neighboring_positions(pos)
+            .possible_queen_bee_move_paths(pos) // seed initial steps from queen bee's paths
             .into_iter()
-            .filter(|&p| self.get_cell(p).is_none())
-            .filter(|&p| {
-                self.cell_neighbors(p)
-                    .filter(|&cell| cell.position != pos)
-                    .count()
-                    != 0
-            })
+            .map(|path| *path.last().unwrap())
             .map(|p| {
                 let mut path = vec![pos, p];
                 path.extend(OutlierNeighborIter::new(self, p, vec![pos, p]));
@@ -578,16 +585,21 @@ impl Hive {
             .collect();
         possible_paths.sort_unstable_by_key(|p| p.len());
 
-        let mut end_poss = HashSet::new();
-
         possible_paths
+    }
+
+    fn possible_soldier_ant_move_paths(&self, pos: Position) -> Vec<Vec<Position>> {
+        let candidates = self.possible_soldier_ant_move_paths_with_duplicates(pos);
+
+        let mut end_poss = HashSet::new();
+        candidates
             .into_iter()
             .filter(|p| end_poss.insert(*p.last().unwrap()))
             .collect()
     }
 
     fn possible_spider_move_paths(&self, pos: Position) -> Vec<Vec<Position>> {
-        self.possible_soldier_ant_move_paths(pos)
+        self.possible_soldier_ant_move_paths_with_duplicates(pos)
             .into_iter()
             .filter(|p| p.len() == 4)
             .collect()
@@ -682,6 +694,44 @@ mod tests {
         );
     }
 
+    #[test]
+    fn beetle_movement2() {
+        let mut hive = Hive {
+            cells: vec![
+                Cell::new(0, Piece::new(Color::Black, Species::QueenBee)),
+                Cell::new(3, Piece::new(Color::Black, Species::Beetle)),
+                Cell::new(6, Piece::new(Color::White, Species::QueenBee)),
+            ],
+        };
+        hive.cells[2]
+            .pieces
+            .push(Piece::new(Color::White, Species::Beetle));
+
+        // valid moves
+        hive.can_move(&[3, 0]).unwrap();
+        hive.can_move(&[3, 2]).unwrap();
+        hive.can_move(&[3, 4]).unwrap();
+    }
+
+    #[test]
+    fn beetle_movement3() {
+        let mut hive = Hive {
+            cells: vec![
+                Cell::new(0, Piece::new(Color::Black, Species::QueenBee)),
+                Cell::new(2, Piece::new(Color::Black, Species::Beetle)),
+                Cell::new(6, Piece::new(Color::White, Species::QueenBee)),
+            ],
+        };
+        hive.cells[0]
+            .pieces
+            .push(Piece::new(Color::White, Species::Beetle));
+
+        // valid moves
+        for i in 1..6 {
+            hive.can_move(&[0, i]).unwrap();
+        }
+    }
+
     #[test]
     fn grasshopper_movement() {
         let hive = Hive {
@@ -709,9 +759,12 @@ mod tests {
         assert_eq!(hive.possible_grasshopper_move_paths(14).len(), 3);
 
         // some invalid moves
-        assert_eq!(hive.can_move(&[14, 18]), Err(MoveError::GrasshopperBadHop));
         assert_eq!(
-            hive.can_move(&[14, 55]),
+            hive.can_move(&[14, 5, 6, 18]),
+            Err(MoveError::GrasshopperBadHop)
+        );
+        assert_eq!(
+            hive.can_move(&[14, 15, 32, 55]),
             Err(MoveError::NotAdjacentToAnotherPiece)
         );
     }
@@ -742,16 +795,8 @@ mod tests {
         // some invalid moves
         assert_eq!(hive.can_move(&[14, 13]), Err(MoveError::CantClimb));
         assert_eq!(hive.can_move(&[14, 15]), Err(MoveError::CantClimb));
-        assert_eq!(hive.can_move(&[14, 28]), Err(MoveError::TooLong));
-        assert_eq!(hive.can_move(&[14, 31]), Err(MoveError::TooLong));
-        assert_eq!(
-            hive.can_move(&[14, 50]),
-            Err(MoveError::NotAdjacentToAnotherPiece)
-        );
-        assert_eq!(
-            hive.can_move(&[14, 51]),
-            Err(MoveError::NotAdjacentToAnotherPiece)
-        );
+        assert_eq!(hive.can_move(&[14, 28]), Err(MoveError::Inconsistent));
+        assert_eq!(hive.can_move(&[14, 30, 31]), Err(MoveError::TooLong));
     }
 
     #[test]
@@ -869,6 +914,49 @@ mod tests {
         );
     }
 
+    #[test]
+    fn spider_movement2() {
+        let hive = Hive {
+            cells: vec![
+                Cell::new(0, Piece::new(Color::Black, Species::QueenBee)),
+                Cell::new(2, Piece::new(Color::White, Species::QueenBee)),
+                Cell::new(4, Piece::new(Color::Black, Species::Grasshopper)),
+                Cell::new(9, Piece::new(Color::White, Species::Grasshopper)),
+                Cell::new(13, Piece::new(Color::Black, Species::Spider)),
+                Cell::new(21, Piece::new(Color::White, Species::Spider)),
+            ],
+        };
+
+        // valid moves
+        hive.can_move(&[13, 14, 5, 6]).unwrap();
+        hive.can_move(&[13, 12, 3, 10]).unwrap();
+    }
+
+    #[test]
+    fn spider_movement3() {
+        let hive = Hive {
+            cells: vec![
+                Cell::new(3, Piece::new(Color::Black, Species::Spider)),
+                Cell::new(4, Piece::new(Color::Black, Species::QueenBee)),
+                Cell::new(5, Piece::new(Color::White, Species::Grasshopper)),
+                Cell::new(6, Piece::new(Color::White, Species::Spider)),
+                Cell::new(15, Piece::new(Color::White, Species::Beetle)),
+                Cell::new(32, Piece::new(Color::Black, Species::SoldierAnt)),
+                Cell::new(33, Piece::new(Color::Black, Species::Grasshopper)),
+                Cell::new(17, Piece::new(Color::White, Species::QueenBee)),
+                Cell::new(18, Piece::new(Color::White, Species::SoldierAnt)),
+            ],
+        };
+
+        // valid moves
+        hive.can_move(&[6, 0, 2, 10]).unwrap();
+        hive.can_move(&[6, 1, 7, 36]).unwrap();
+
+        // invalid moves
+        hive.can_move(&[6, 1, 0, 2]).unwrap_err(); // one hive rule broken between 1 and 0
+        hive.can_move(&[6, 0, 2, 1]).unwrap_err(); // one hive rule broken between 2 and 1
+    }
+
     #[test]
     fn no_gap_jumping() {
         let hive = Hive {
